import ast
from pathlib import Path
from logging import debug
from transpiler.phases.typing.scope import VarKind, VarDecl, ScopeKind, Scope
from transpiler.phases.typing.stdlib import PRELUDE, StdlibVisitor
from transpiler.phases.typing.types import TY_TYPE, TY_INT, TY_STR, TY_BOOL, TY_COMPLEX, TY_NONE, FunctionType, \
    TypeVariable, CppType, PyList, TypeType, Forked, Task, Future, PyIterator, TupleType, TypeOperator, BaseType, \
    ModuleType, TY_BYTES, TY_FLOAT, PyDict, TY_SLICE, TY_OBJECT, BuiltinFeature, UnionType

PRELUDE.vars.update({
    # "int": VarDecl(VarKind.LOCAL, TY_TYPE, TY_INT),
    # "str": VarDecl(VarKind.LOCAL, TY_TYPE, TY_STR),
    # "bool": VarDecl(VarKind.LOCAL, TY_TYPE, TY_BOOL),
    # "complex": VarDecl(VarKind.LOCAL, TY_TYPE, TY_COMPLEX),
    # "None": VarDecl(VarKind.LOCAL, TY_NONE, None),
    # "Callable": VarDecl(VarKind.LOCAL, TY_TYPE, FunctionType),
    # "TypeVar": VarDecl(VarKind.LOCAL, TY_TYPE, TypeVariable),
    # "CppType": VarDecl(VarKind.LOCAL, TY_TYPE, CppType),
    # "list": VarDecl(VarKind.LOCAL, TY_TYPE, PyList),
    "int": VarDecl(VarKind.LOCAL, TypeType(TY_INT)),
    "float": VarDecl(VarKind.LOCAL, TypeType(TY_FLOAT)),
    "str": VarDecl(VarKind.LOCAL, TypeType(TY_STR)),
    "bytes": VarDecl(VarKind.LOCAL, TypeType(TY_BYTES)),
    "bool": VarDecl(VarKind.LOCAL, TypeType(TY_BOOL)),
    "complex": VarDecl(VarKind.LOCAL, TypeType(TY_COMPLEX)),
    "None": VarDecl(VarKind.LOCAL, TypeType(TY_NONE)),
    "Callable": VarDecl(VarKind.LOCAL, TypeType(FunctionType)),
    "TypeVar": VarDecl(VarKind.LOCAL, TypeType(TypeVariable)),
    "CppType": VarDecl(VarKind.LOCAL, TypeType(CppType)),
    "list": VarDecl(VarKind.LOCAL, TypeType(PyList)),
    "dict": VarDecl(VarKind.LOCAL, TypeType(PyDict)),
    "Forked": VarDecl(VarKind.LOCAL, TypeType(Forked)),
    "Task": VarDecl(VarKind.LOCAL, TypeType(Task)),
    "Future": VarDecl(VarKind.LOCAL, TypeType(Future)),
    "Iterator": VarDecl(VarKind.LOCAL, TypeType(PyIterator)),
    "tuple": VarDecl(VarKind.LOCAL, TypeType(TupleType)),
    "slice": VarDecl(VarKind.LOCAL, TypeType(TY_SLICE)),
    "object": VarDecl(VarKind.LOCAL, TypeType(TY_OBJECT)),
    "BuiltinFeature": VarDecl(VarKind.LOCAL, TypeType(BuiltinFeature)),
    "Any": VarDecl(VarKind.LOCAL, TypeType(TY_OBJECT)),
    "Optional": VarDecl(VarKind.LOCAL, TypeType(lambda x: UnionType(x, TY_NONE))),
})

typon_std = Path(__file__).parent.parent.parent.parent / "stdlib"

def make_module(name: str, scope: Scope) -> BaseType:
    ty = ModuleType([], f"{name}")
    for n, v in scope.vars.items():
        ty.members[n] = v.type
    return ty


def discover_module(path: Path, scope):
    for child in sorted(path.iterdir()):
        if child.is_dir():
            mod_scope = PRELUDE.child(ScopeKind.GLOBAL)
            discover_module(child, mod_scope)
            scope.vars[child.name] = make_mod_decl(child.name, mod_scope)
        elif child.name == "__init__.py":
            StdlibVisitor(scope).visit(ast.parse(child.read_text()))
            debug(f"Visited {child}")
        elif child.suffix == ".py":
            mod_scope = PRELUDE.child(ScopeKind.GLOBAL)
            StdlibVisitor(mod_scope).visit(ast.parse(child.read_text()))
            if child.stem[-1] == "_":
                child = child.with_name(child.stem[:-1])
            scope.vars[child.stem] = make_mod_decl(child.name, mod_scope)
            debug(f"Visited {child}")


def make_mod_decl(child, mod_scope):
    return VarDecl(VarKind.MODULE, make_module(child, mod_scope), {k: v.type for k, v in mod_scope.vars.items()})


discover_module(typon_std, PRELUDE)

debug("Stdlib visited!")
#exit()